#include <avr/interrupt.h>
#include <avr/io.h>

#include <util/delay.h>

#include <stdio.h>
#include <stdlib.h>

#define RED 1
#define GREEN 2
#define BLUE 3
#define OFF 0
#define NRM 4
#define NRD 16

unsigned int models[][30*7] = { {0, 0, 3, 3, 3, 0, 3, 0, 0, 0, 3, 0, 2, 2, 2, 0, 2, 2, 2, 0, 2, 0, 2, 2, 2, 2, 0, 0, 0, 0,
							 0, 0, 3, 0, 3, 0, 3, 3, 0, 3, 3, 0, 0, 0, 2, 0, 2, 0, 2, 0, 3, 0, 0, 0, 0, 2, 0, 0, 0, 0,
							 0, 0, 3, 0, 3, 0, 3, 0, 3, 0, 3, 0, 0, 0, 2, 0, 2, 0, 2, 0, 1, 0, 0, 0, 0, 2, 0, 0, 0, 0,
							 0, 0, 3, 3, 3, 0, 3, 0, 0, 0, 3, 0, 0, 2, 0, 0, 2, 0, 2, 0, 2, 0, 0, 0, 0, 2, 0, 0, 0, 0,
							 0, 0, 3, 0, 0, 0, 3, 0, 0, 0, 3, 0, 2, 0, 0, 0, 2, 0, 2, 0, 3, 0, 0, 0, 0, 2, 0, 0, 0, 0,
							 0, 0, 3, 0, 0, 0, 3, 0, 0, 0, 3, 0, 2, 0, 0, 0, 2, 0, 2, 0, 1, 0, 0, 0, 0, 2, 0, 0, 0, 0,
							 0, 0, 3, 0, 0, 0, 3, 0, 0, 0, 3, 0, 2, 2, 2, 0, 2, 2, 2, 0, 2, 0, 0, 0, 0, 2, 0, 0, 0, 0},
							 
							{1, 1, 1, 1, 2, 2, 2, 3, 3, 0, 2, 0, 2, 0, 3, 0, 0, 2, 0, 2, 0, 3, 3, 2, 2, 2, 1, 1, 1, 1,
							 1, 1, 1, 1, 2, 2, 2, 3, 3, 0, 0, 2, 0, 3, 0, 3, 0, 0, 2, 0, 0, 3, 3, 2, 2, 2, 1, 1, 1, 1,
							 1, 1, 1, 1, 2, 2, 2, 3, 3, 0, 0, 0, 3, 0, 0, 0, 3, 0, 0, 0, 0, 3, 3, 2, 2, 2, 1, 1, 1, 1,
							 1, 1, 1, 1, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 1, 1, 1, 1,
							 1, 1, 1, 1, 2, 2, 2, 3, 3, 0, 0, 0, 3, 0, 0, 0, 3, 0, 0, 0, 0, 3, 3, 2, 2, 2, 1, 1, 1, 1,
							 1, 1, 1, 1, 2, 2, 2, 3, 3, 0, 0, 2, 0, 3, 0, 3, 0, 0, 2, 0, 0, 3, 3, 2, 2, 2, 1, 1, 1, 1,
							 1, 1, 1, 1, 2, 2, 2, 3, 3, 0, 2, 0, 2, 0, 3, 0, 0, 2, 0, 2, 0, 3, 3, 2, 2, 2, 1, 1, 1, 1},
							 
							{0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
							 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
							 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0,
							 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0,
							 1, 1, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0,
							 1, 1, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
							 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1},
							 
							{0, 0, 0, 0, 0, 0, 0, 2, 2, 0, 3, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0,
							 0, 0, 3, 0, 0, 0, 0, 2, 0, 3, 3, 3, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 2,
							 0, 3, 3, 3, 0, 0, 0, 0, 3, 3, 1, 3, 3, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 2, 2,
							 3, 3, 1, 3, 3, 0, 0, 0, 0, 3, 3, 3, 0, 0, 0, 3, 1, 2, 1, 3, 3, 3, 1, 2, 1, 1, 3, 2, 2, 2,
							 0, 3, 3, 3, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 2, 2,
							 2, 0, 3, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 2,
							 2, 2, 0, 2, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
						};
unsigned int ind;
unsigned int j;
unsigned int state;
unsigned int next;

/* initializeaza timer-ele*/
void timer_init()
{
    // pragul initial de comparare
    OCR1A = 0x3D09;
    OCR0A = 0xC8;
    // activeaza intreruperea de comparare cu OCR1A
    TIMSK1 |= (1 << OCIE1A);
    TIMSK0 |= (1 << OCIE0A);
    // modul CTC cu OCR1A
    TCCR1B |= (5 << CS10) | (1 << WGM12);
	TCCR0A |= (2 << WGM00);
	
	//setare prescaler
	TCCR1B |= (3 << CS10);
	TCCR0B |= (3 << CS00);
	
}

/* intreruperea de comparatie cu OCR0A */
ISR(TIMER0_COMPA_vect)
{

	unsigned int x =0;
	PORTD &= ~(3 << PD0);
    PORTB &= ~(15 << PB0);
	PORTC = 0;
	PORTA = 0;
	
	x = models[ind][j*8];
	if(x == RED){
		PORTA |= (1 << PA5);
	}
	else if(x == GREEN){
		PORTA |= (1 << PA6);
	}
	else if(x == BLUE){
		PORTA |= (1 << PA7);
	}

	x = models[ind][j*8 + 1];
	if(x == RED){
		PORTA |= (1 << PA2);
	}
	else if(x == GREEN){
		PORTA |= (1 << PA3);
	}
	else if(x == BLUE){
		PORTA |= (1 << PA4);
	}
	
	x = models[ind][j*8 + 2];
	if(x == RED){
		PORTC |= (1 << PC6);
	}
	else if(x == GREEN){
		PORTA |= (1 << PA0);
	}
	else if(x == BLUE){
		PORTA |= (1 << PA1);
	}
	
	x = models[ind][j*8 + 3];
	if(x == RED){
		PORTC |= (1 << PC3);
	}
	else if(x == GREEN){
		PORTC |= (1 << PC4);
	}
	else if(x == BLUE){
		PORTC |= (1 << PC5);
	}
	x = models[ind][j*8 + 4];
	if(x == RED){
		PORTC |= (1 << PC0);
	}
	else if(x == GREEN){
		PORTC |= (1 << PC1);
	}
	else if(x == BLUE){
		PORTC |= (1 << PC2);
	}
	
	x = models[ind][j*8 + 5];
	if(x == RED){
		PORTB |= (1 << PB0);
	}
	else if(x == GREEN){
		PORTD |= (1 << PD1);
	}
	else if(x == BLUE){
		PORTD |= (1 << PD0);
	}
	
	x = models[ind][j*8 + 6];
	if(x == RED){
		PORTB |= (1 << PB3);
	}
	else if(x == GREEN){
		PORTB |= (1 << PB2);
	}
	else if(x == BLUE){
		PORTB |= (1 << PB1);
	}
	
	j = (j + 1) % NRD;
}

/* intreruperea pentru pin change*/
ISR(PCINT3_vect)
{
    
    // PD6 apasat
    if((PIND & (1 << PD6)) == 0)
    {
		state ^= 1;
		if(state){
			PORTB |= (1 << PB4);
			PORTD &= ~(1 << PD7);
			_delay_ms(5000);
			timer_init();
		}
		else {
			PORTD &= ~(3 << PD0);
			PORTB &= ~(15 << PB0);
			PORTC = 0;
			PORTA = 0;
			TIMSK1 &= ~(1 << OCIE1A);
			TIMSK0 &= ~(1 << OCIE0A);
			PORTD |= (1 << PD7);
			PORTB |= (1 << PB4);
			PORTB |= (1 << PB0);
			PORTD |= (1 << PD0);
			PORTD |= (1 << PD1);
			PORTB &= ~(1 << PB4);
			
		}
    }
	
}

/* initializeaza intrarile si iesirile */
ISR(TIMER1_COMPA_vect){
	next++;
	if(next == 10){
		ind = (ind + 1) % NRM;
		next = 0;
	}
}
	
void IO_init()
{
    // LED-ul RGB este pus ca iesire
	ind = 0;
    DDRD |= (3 << PD0);
	DDRD |= (1 << PD7);
    DDRB |= (31 << PB0);
	DDRA |= 255;
	DDRC |= 255;
    
    // butoanele sunt puse ca intrare, cu rezistenta de pull-up
    DDRD &= ~(1 << PD6);
    PORTD |= (1 << PD6);
    
    // intreruperile sunt activate pentru PCINT30 (PD6)
    PCICR  |= (1 << PCIE3);
    PCMSK3 |= (1 << PCINT30);
}



int main(void)
{
	state = 0;
	next = 0;
    sei();
	
    IO_init();
	
	while(1);

    return 0;
}



















